package Hessian::Simple;

use Moose;


"one, but we're not the same";

__END__


=head1 NAME

Hessian::Simple - Experimenting with dynamic fields

=head1 VERSION

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 INTERFACE


